// core/init.cc -- C++ entry point and component initialization.
//
// This software is copyright (c) 2006 Scott Wood <scott@buserror.net>.
// 
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors or contributors be held liable for any damages
// arising from the use of this software.
// 
// Permission is hereby granted to everyone, free of charge, to use, copy,
// modify, prepare derivative works of, publish, distribute, perform,
// sublicense, and/or sell copies of the Software, provided that the above
// copyright notice and disclaimer of warranty be included in all copies or
// substantial portions of this software.

#include <kern/types.h>
#include <kern/libc.h>
#include <kern/console.h>
#include <kern/arch.h>
#include <kern/time.h>
#include <kern/thread.h>
#include <kern/orb.h>

extern void *eh_frame_begin;
extern "C" void __register_frame(const void *begin);

void run_test();

extern "C" void start_kernel()
{
	run_ctors();
	Arch::arch_init();

	// __register_frame must not be called until after dynamic memory
	// allocation is initialized (exceptions wouldn't work before then,
	// anyway; if one is thrown, abort() will be called).
	
	__register_frame(&eh_frame_begin);
	printf("Starting kernel...\n");

	Time::init();
	Threads::sched.init();
	ll_ints_on();
	ORB::init();
	
	run_test();
	
	printf("done.\n");
	for(;;);
}
