#ifndef _KERN_SPINLOCK_H
#define _KERN_SPINLOCK_H

#include <kern/kernel.h>
#include <lowlevel/misc.h>

extern int dying;

namespace Lock {
	struct SpinLock {
		#if CONF_DEBUG_SPINLOCKS
			unsigned long held_at;
		#endif	
	
		SpinLock()
		{
			init();
		}

		void init()
		{
			// Nothing on UP.  Call arch on SMP.
		
			#if CONF_DEBUG_SPINLOCKS
				held_at = 0;
			#endif
		}

		void lock()
		{
			// Nothing on UP.  Call arch on SMP.

			#if CONF_DEBUG_SPINLOCKS
				#if !CONF_SMP
					if (!in_fault && held_at) {
						in_fault++;
						printf("Spinlock deadlock, lock %p previously held at 0x%lx\n",
						       this, held_at);
						in_fault--;
						BUG();
					}
				#endif

				held_at = (ulong)__builtin_return_address(0);
			#endif
		}
		
		void unlock()
		{
			#if CONF_DEBUG_SPINLOCKS
				held_at = 0;
			#endif

			// Nothing on UP.  Call arch on SMP.
		}

		void lock_irq()
		{
			ll_ints_off();
			lock();
		}

		void unlock_irq()
		{
			unlock();
			ll_ints_on();
		}
		
		ulong lock_recirq()
		{
			unsigned long ret = ll_ints_save_and_off();
			lock();
			return ret;
		}
		
		void unlock_recirq(ulong savedirq)
		{
			unlock();
			ll_ints_restore(savedirq);
		}
	};
}

#endif
